@@ -124,6 +124,13 @@ static int parse_xml(const char *basedir, xmlNode *root_node, struct ploop_disk_
 			di->size = val;
 	}
 
+	node = seek(cur_node, "Max_delta_size");
+	if (node != NULL) {
+		data = get_element_txt(node);
+		if (parse_ul(data, &val) == 0)
+			di->max_delta_size = val;
+	}
+
 	node = seek(cur_node, "Cylinders");
 	if (node != NULL) {
 		data = get_element_txt(node);
@@ -436,14 +443,16 @@ int ploop_read_disk_descr(struct ploop_disk_images_data **di, const char *file)
 	return ploop_read_dd(*di);
 }
 
-static int normalize_image_name(const char *basedir, const char *image, char *out, int len)
+int normalize_image_name(const char *basedir, const char *image, char *out, int len)
 {
 	const char *p;
 	int n;
 
 	n = strlen(basedir);
+	while (basedir[n-1] == '/')
+		n--;
 	p = image;
-	if (strncmp(image, basedir, n) == 0)
+	if ((strncmp(image, basedir, n) == 0) && (image[n] == '/'))
 		p += n + 1;
 
 	snprintf(out, len, "%s", p);
@@ -517,6 +526,15 @@ int ploop_store_diskdescriptor(const char *fname, struct ploop_disk_images_data
 		goto err;
 	}
 
+	if (di->max_delta_size != 0) {
+		rc = xmlTextWriterWriteFormatElement(writer,
+				BAD_CAST "Max_delta_size", "%llu", di->max_delta_size);
+		if (rc < 0) {
+			ploop_err(0, "Error at xmlTextWriter Max_delta_size");
+			goto err;
+		}
+	}
+
 	rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST "Cylinders", "%u", di->cylinders);
 	if (rc < 0) {
 		ploop_err(0, "Error at xmlTextWriter Cylinders");
