@@ -22,9 +22,7 @@
 #include <stdio.h>
 #include <signal.h>
 
-#include "ploop_if.h"
-#include "ploop1_image.h"
-#include "libploop.h"
+#include "ploop.h"
 
 int parse_size(const char *opt, off_t *sz, const char *name)
 {
@@ -41,27 +39,27 @@ int parse_size(const char *opt, off_t *sz, const char *name)
 
 	switch (*endptr) {
 	case 'T': case 't':
-		if (val >= ~0ULL/(1024ULL*1024*1024*1024/512))
+		if (val >= ~0ULL/(1024ULL*1024*1024*1024/SECTOR_SIZE))
 			goto large;
-		val *= 1024ULL*1024*1024*1024/512;
+		val *= 1024ULL*1024*1024*1024/SECTOR_SIZE;
 		*sz = val;
 		break;
 	case 'G': case 'g':
-		if (val >= ~0ULL/(1024*1024*1024/512))
+		if (val >= ~0ULL/(1024*1024*1024/SECTOR_SIZE))
 			goto large;
-		val *= 1024*1024*1024/512;
+		val *= 1024*1024*1024/SECTOR_SIZE;
 		*sz = val;
 		break;
 	case 'M': case 'm':
-		if (val >= ~0ULL/(1024*1024/512))
+		if (val >= ~0ULL/(1024*1024/SECTOR_SIZE))
 			goto large;
-		val *= 1024*1024/512;
+		val *= 1024*1024/SECTOR_SIZE;
 		*sz = val;
 		break;
 	case 'K': case 'k':
-		if (val >= ~0ULL/(1024/512))
+		if (val >= ~0ULL/(1024/SECTOR_SIZE))
 			goto large;
-		val *= 1024/512;
+		val *= 1024/SECTOR_SIZE;
 		*sz = val;
 		break;
 	case 0:
@@ -99,6 +97,29 @@ int parse_format_opt(const char *opt)
 	return -1;
 }
 
+char *parse_uuid(const char *opt)
+{
+	char buf[] = "{fbcdf284-5345-416b-a589-7b5fcaa87673}";
+	const char *id = opt;
+
+	if (!id)
+		goto err;
+	if (id[0] != '{' && strlen(id) == 36) {
+		/* as a courtesy, add missing brackets */
+		memcpy(buf+1, id, 36);
+		id = buf;
+	}
+	if (!is_valid_guid(id))
+		goto err;
+
+	return strdup(id);
+
+err:
+	fprintf(stderr, "Incorrect uuid specified: %s\n", opt);
+	return NULL;
+
+}
+
 int is_xml_fname(const char *fname)
 {
 	const char *p;
