@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2008-2012, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2008-2015, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include <sys/types.h>
 #include <dirent.h>
 #include <ctype.h>
+#include <getopt.h>
 
 #include "ploop.h"
 #include "common.h"
@@ -55,10 +56,10 @@ static void usage_summary(void)
 			"       ploop balloon { show | status | clear | change | complete | check |\n"
 			"                       repair | discard } ... DiskDescriptor.xml\n"
 			"       ploop snapshot DiskDescriptor.xml\n"
-			"       ploop snapshot-delete -u <uuid> DiskDescriptor.xml\n"
-			"       ploop snapshot-merge [-u <uuid>] DiskDescriptor.xml\n"
-			"       ploop snapshot-switch -u <uuid> DiskDescriptor.xml\n"
-			"       ploop snapshot-list [-o field[,field...]] [-u <UUID>] DiskDescriptor.xml\n"
+			"       ploop snapshot-delete -u UUID DiskDescriptor.xml\n"
+			"       ploop snapshot-merge [-u UUID] DiskDescriptor.xml\n"
+			"       ploop snapshot-switch -u UUID DiskDescriptor.xml\n"
+			"       ploop snapshot-list [-o field[,field...]] [-u UUID] DiskDescriptor.xml\n"
 			"Also:  ploop { start | stop | delete | clear | merge | grow | copy |\n"
 			"               stat | info | list} ...\n"
 			"\n"
@@ -578,7 +579,7 @@ static int plooptool_rm(int argc, char **argv)
 
 static void usage_snapshot(void)
 {
-	fprintf(stderr, "Usage: ploop snapshot [-u <uuid>] DiskDescriptor.xml\n"
+	fprintf(stderr, "Usage: ploop snapshot [-u UUID] DiskDescriptor.xml\n"
 			"       ploop snapshot [-F] -d DEVICE DELTA\n"
 			"       DEVICE := ploop device, e.g. /dev/ploop0\n"
 			"       DELTA := path to new image file\n"
@@ -642,8 +643,8 @@ static int plooptool_snapshot(int argc, char **argv)
 
 static void usage_tsnapshot(void)
 {
-	fprintf(stderr, "Usage: ploop tsnapshot -u <uuid> -c <component_name>\n"
-			"	[-m MOUNT_POINT] DiskDescriptor.xml\n"
+	fprintf(stderr, "Usage: ploop tsnapshot -u UUID -c COMPONENT\n"
+			"       [-m MOUNT_POINT] DiskDescriptor.xml\n"
 		);
 }
 
@@ -667,7 +668,7 @@ static int plooptool_tsnapshot(int argc, char **argv)
 			param.target = optarg;
 			break;
 		default:
-			usage_snapshot();
+			usage_tsnapshot();
 			return SYSEXIT_PARAM;
 		}
 	}
@@ -695,8 +696,8 @@ static int plooptool_tsnapshot(int argc, char **argv)
 
 static void usage_snapshot_switch(void)
 {
-	fprintf(stderr, "Usage: ploop snapshot-switch -u <uuid> DiskDescriptor.xml\n"
-			"       -u <uuid>     snapshot uuid\n");
+	fprintf(stderr, "Usage: ploop snapshot-switch -u UUID DiskDescriptor.xml\n"
+			"       UUID := snapshot UUID\n");
 }
 
 static int plooptool_snapshot_switch(int argc, char **argv)
@@ -744,8 +745,8 @@ static int plooptool_snapshot_switch(int argc, char **argv)
 
 static void usage_snapshot_delete(void)
 {
-	fprintf(stderr, "Usage: ploop snapshot-delete -u <uuid> DiskDescriptor.xml\n"
-			"       -u <uuid>     snapshot uuid\n");
+	fprintf(stderr, "Usage: ploop snapshot-delete -u UUID DiskDescriptor.xml\n"
+			"       UUID := snapshot id\n");
 }
 
 static int plooptool_snapshot_delete(int argc, char **argv)
@@ -788,8 +789,9 @@ static int plooptool_snapshot_delete(int argc, char **argv)
 
 static void usage_snapshot_merge(void)
 {
-	fprintf(stderr, "Usage: ploop snapshot-merge [-u <uuid> | -A] DiskDescriptor.xml\n"
-			"       -u <uuid>     snapshot uuid (top delta if not specified)\n");
+	fprintf(stderr, "Usage: ploop snapshot-merge [-u UUID | -A] [-n DELTA] DiskDescriptor.xml\n"
+			"       -u UUID       snapshot to merge (top delta if not specified)\n"
+			"       -n DELTA      new delta file to merge to\n");
 }
 
 static int plooptool_snapshot_merge(int argc, char ** argv)
@@ -797,7 +799,7 @@ static int plooptool_snapshot_merge(int argc, char ** argv)
 	int i, ret;
 	struct ploop_merge_param param = {};
 
-	while ((i = getopt(argc, argv, "u:A")) != EOF) {
+	while ((i = getopt(argc, argv, "u:n:A")) != EOF) {
 		switch (i) {
 		case 'u':
 			param.guid = parse_uuid(optarg);
@@ -807,6 +809,9 @@ static int plooptool_snapshot_merge(int argc, char ** argv)
 		case 'A':
 			param.merge_all = 1;
 			break;
+		case 'n':
+			param.new_delta = strdup(optarg);
+			break;
 		default:
 			usage_snapshot_merge();
 			return SYSEXIT_PARAM;
@@ -1141,29 +1146,35 @@ static int plooptool_list(int argc, char **argv)
 
 static void usage_replace(void)
 {
-	fprintf(stderr, "Usage: ploop replace {-d DEVICE | -m MNT} {-l LVL | -o СDELTA} -i DELTA\n"
-			"       ploop replace {-u UUID | -l LVL | -o CDELTA} -i DELTA DiskDescriptor.xml\n"
+	fprintf(stderr, "Usage: ploop replace {-d DEVICE | -m MNT} {-l LVL | -o СDELTA} [-k] -i DELTA\n"
+			"       ploop replace {-u UUID|-l LVL|-o CDELTA} [-k] -i DELTA DiskDescriptor.xml\n"
 			"       DEVICE := ploop device, e.g. /dev/ploop0\n"
 			"       MNT := directory where ploop is mounted to\n"
 			"       LVL := NUMBER, distance from base delta\n"
 			"       UUID := UUID of image to be replaced\n"
 			"       CDELTA := path to currently used image file\n"
 			"       DELTA := path to new image file\n"
+			"       -k, --keep-name    keep the file name (rename DELTA to CDELTA)\n"
 	       );
 }
 
 
 static int plooptool_replace(int argc, char **argv)
 {
-	int i;
+	int i, idx;
 	char dev[PATH_MAX];
 	char *device = NULL;
 	char *mnt = NULL;
 	struct ploop_replace_param param = {
 		.level = -1,
 	};
+	static struct option options[] = {
+		{"keep-name", no_argument, NULL, 'k'},
+		{NULL, 0, NULL, 0 }
+	};
 
-	while ((i = getopt(argc, argv, "d:m:l:i:u:o:")) != EOF) {
+	while ((i = getopt_long(argc, argv, "d:m:l:i:u:o:k",
+					options, &idx)) != EOF) {
 		switch (i) {
 		case 'd':
 			device = optarg;
@@ -1185,6 +1196,9 @@ static int plooptool_replace(int argc, char **argv)
 		case 'o':
 			param.cur_file = strdup(optarg);
 			break;
+		case 'k':
+			param.flags |= PLOOP_REPLACE_KEEP_NAME;
+			break;
 		default:
 			usage_replace();
 			return SYSEXIT_PARAM;
@@ -1256,12 +1270,13 @@ static int plooptool_replace(int argc, char **argv)
 		if (param.cur_file) {
 			int ret;
 
-			level = find_level_by_delta(device, param.cur_file);
-			if (level < 0) {
+			ret = find_level_by_delta(device, param.cur_file,
+					&level);
+			if (ret) {
 				fprintf(stderr, "Can't find level by "
 						"delta file name %s",
 						param.cur_file);
-				return SYSEXIT_PARAM;
+				return ret;
 			}
 
 			ret = check_deltas_same(param.cur_file, param.file);
@@ -1273,6 +1288,57 @@ static int plooptool_replace(int argc, char **argv)
 	}
 }
 
+static void usage_restore_descriptor(void)
+{
+	fprintf(stderr, "Usage: ploop restore-descriptor [-f FORMAT] [-b BLOCKSIZE] IMAGE_DIR BASE_DELTA\n"
+			"       FORMAT := { raw | ploop1 }\n"
+			"       BLOCKSIZE := block size in sectors (for raw image format)"
+			"       *IMAGE_DIR := directory where to place DiskDescriptor.xml\n"
+			"       *BASE_DELTA := path to image file\n"
+		);
+}
+
+static int plooptool_restore_descriptor(int argc, char **argv)
+{
+	int i, f;
+	int raw = 0;
+	int blocksize = 0;
+	char *endptr;
+
+	while ((i = getopt(argc, argv, "f:b:")) != EOF) {
+		switch (i) {
+		case 'f':
+			f = parse_format_opt(optarg);
+			if (f < 0) {
+				usage_restore_descriptor();
+				return SYSEXIT_PARAM;
+			}
+			raw = (f == PLOOP_RAW_MODE);
+			break;
+		case 'b':
+			  blocksize = strtoul(optarg, &endptr, 0);
+			  if (*endptr != '\0') {
+				  usage_restore_descriptor();
+				  return SYSEXIT_PARAM;
+			  }
+			  break;
+		default:
+			  usage_restore_descriptor();
+			  return SYSEXIT_PARAM;
+		}
+	}
+
+	argc -= optind;
+	argv += optind;
+
+	if (argc != 2) {
+		usage_restore_descriptor();
+		return SYSEXIT_PARAM;
+	}
+
+	return ploop_restore_descriptor(argv[0], argv[1], raw, blocksize);
+}
+
 int main(int argc, char **argv)
 {
 	char * cmd;
@@ -1369,6 +1435,8 @@ int main(int argc, char **argv)
 		return plooptool_copy(argc, argv);
 	if (strcmp(cmd, "replace") == 0)
 		return plooptool_replace(argc, argv);
+	if (strcmp(cmd, "restore-descriptor") == 0)
+		return plooptool_restore_descriptor(argc, argv);
 
 	if (cmd[0] != '-') {
 		char ** nargs;
